/*
 * Memory trainiert das Kurzzeitgedaechnis.
 * Der Computer wrfelt verdeckt vier Buchstaben
 * aus der Buchstabenmenge A, B, C, D, E und F.
 * Wiederholungen sind dabei erlaubt.
 * Der Spieler soll Buchstaben und Reihenfolge erraten.
 * w bedeutet: richtiger Buchstaben an richtiger Stelle
 * s Bedeutet: richtiger Buchstaben an falscher Stelle
 */
# include <stdio.h>
# include <stdlib.h>
# include <time.h>
# include "matrix.h"

# define BREITE 4

/*
 * Erzeugt Brett als Matrix unter Verwendung
 * der Funktionen aus matrix.h, legt Karten auf
 * und mischt diese.
 * Parameter: Spielbrett, Brettbreite
 */
void initBrett( double **, int);

/*
 * Ziehen zweier Karten.
 * Parameter: Spielbrett, Brettbreite
 * Rueckgabe: 1 .. Karten identisch, 0 .. sonst
 */
int ziehe( double **, int);

/*
 * Zudecken aller Karten.
 * Parameter: Spielbrett, Brettbreite
 * Rueckgabe: 1 .. keine Karten mehr da - fertig,
 *            0 .. sonst
 */
int deckeZu( double **, int);

/*
 * Aufdecken einer Karten.
 * Parameter: Spielbrett, Brettbreite, Zeile, Spalte
 */
void deckeEineAuf( double **, int, int, int);

/*
 * Aufdecken zweier Karten.
 * Parameter: Spielbrett, Brettbreite,
 *            Zeile, Spalte der ersten Karte
 *            Zeile, Spalte der zweiten Karte
 */
void deckeBeideAuf
     ( double **, int, int, int, int, int);

/*
 * Aufdecken alle Karten.
 * Parameter: Spielbrett, Brettbreite
 */
void zeigeBrett( double **, int);


/*
 * Startet Memory.
 */
int main()
{
  double **brett = NULL;
  char jn[ 2] = "n";
  int runden = 0;
  brett = initMatrix( BREITE, BREITE);

  do
  {
    initBrett( brett, BREITE);
/*    zeigeBrett( brett, BREITE);       Testausdruck */

    do
    {
      printf( "\n");
      if( !deckeZu( brett, BREITE))
      {
        if( ziehe( brett, BREITE)) printf("OK\n");
        runden++;
        printf( "Weiter (j/n)? ");
        scanf( "%1s", jn);
      }
      else
      {
        printf( "\nFERTIG nach %d Runden!\n", runden);
        jn[ 0] = 'n';
      }
    } while( jn[ 0] == 'j');

    printf( "Noch ein Spiel (j/n)? ");
    scanf( "%1s", jn);
  } while( jn[ 0] == 'j');

  return 0;
}

/*
 * Erzeugt Brett als Matrix unter Verwendung
 * der Funktionen aus matrix.h, legt Karten auf
 * und mischt diese.
 */
void initBrett( double **brett, int n)
{
  int i, j, k = 2, i1, j1;
  double temp;;

  for( i = 0; i < n; i++)               /* Auflegen */
    for( j = 0; j < n; j++)
      brett[ i][ j] = k++ / 2;

  srand( time( NULL));
  for( i = 0; i < n; i++)                /* Mischen */
    for( j = 0; j < n; j++)
    {
      i1 = rand() % n;
      j1 = rand() % n;
  
      temp =  brett[ i][ j];
      brett[ i][ j] = brett[ j1][ i1];
      brett[ j1][ i1] = temp;
    }
}

/*
 * Ziehen zweier Karten.
 */
int ziehe( double **brett, int n)
{
  int zeile1 = 0, spalte1 = 0;
  int zeile2 = 0, spalte2 = 0;

  printf( "\n");
/* Eingabe der ersten Karte */
  do
  {
    printf( "Karte 1");
    do
    {
      printf( "\tZeile (0 .. %d):  ", n - 1);
      scanf( "%d", &zeile1);
    } while( zeile1 < 0 || zeile1 >= n);
    do
    {
      printf( "\tSpalte(0 .. %d):  ", n - 1);
      scanf( "%d", &spalte1);
    } while( spalte1 < 0 || spalte1 >= n);
  } while( brett[ zeile1][ spalte1] == 0);

/* Aufdecken der eingegebenen Karte */
  printf( "\n");
  deckeEineAuf( brett, n, zeile1, spalte1);
  printf( "\n");

/* Eingabe der zweiten Karte */
  do
  {
    printf( "Karte 2");
    do
    {
      printf( "\tZeile (0 .. %d):  ", n - 1);
      scanf( "%d", &zeile2);
    } while( zeile2 < 0 || zeile2 >= n);

    do
    {
      printf( "\tSpalte(0 .. %d):  ", n - 1);
      scanf( "%d", &spalte2);
    } while( spalte2 < 0 || spalte2 >= n);
  } while( brett[ zeile2][ spalte2] == 0 ||
           zeile1 == zeile2 && spalte1 == spalte2);

/* Aufdecken beider eingegebenen Karten */
  printf( "\n");
  deckeBeideAuf
  ( brett, n, zeile1, spalte1, zeile2, spalte2);
  printf( "\n");

/* Identische Karten aus dem Spiel nehmen */
  if( brett[ zeile1][ spalte1]
      == brett[ zeile2][ spalte2])
  {
    brett[ zeile1][ spalte1]
    = brett[ zeile2][ spalte2] = 0;
    return 1;
  }

  return 0;
}

/*
 * Zudecken aller Karten.
 */
int deckeZu( double **brett, int n)
{
  int i, j, fertig = 1;

  for( j = 0; j < n; j++) printf( "\t%d", j);
  printf( "\n");
  for( j = 0; j < n; j++) printf( "\t_");
  printf( "\n\n");

  for( i = 0; i < n; i++)
  {
    printf( "%d | ", i);
    for( j = 0; j < n; j++)
      if( (int)brett[ i][ j] != 0) /* Karte im Spiel */
      {
        printf( "\tX");
        fertig = 0;
      }
      else  printf( "\t");
    printf( "\n\n");
  }

  return fertig;
}

/*
 * Aufdecken einer Karten.
 */
void deckeEineAuf
( double **brett, int n, int zeile, int spalte)
{
  int i, j;

  for( j = 0; j < n; j++) printf( "\t%d", j);
  printf( "\n");
  for( j = 0; j < n; j++) printf( "\t_");
  printf( "\n\n");

  for( i = 0; i < n; i++)
  {
    printf( "%d | ", i);
    for( j = 0; j < n; j++)
      if( i == zeile && j == spalte)   /* Aufdecken */
        printf( "\t%d", (int)brett[ i][ j]);
      else
        if( (int)brett[ i][ j] != 0) printf( "\tX");
        else  printf( "\t");
    printf( "\n\n");
  }
}

/*
 * Aufdecken zweier Karten.
 */
void deckeBeideAuf
( double **brett, int n,
  int zeile1, int spalte1, int zeile2, int spalte2)
{
  int i, j;

  for( j = 0; j < n; j++) printf( "\t%d", j);
  printf( "\n");
  for( j = 0; j < n; j++) printf( "\t_");
  printf( "\n\n");

  for( i = 0; i < n; i++)
  {
    printf( "%d | ", i);
    for( j = 0; j < n; j++)            /* Aufdecken */
      if( i == zeile1 && j == spalte1 ||
          i == zeile2 && j == spalte2 )
        printf( "\t%d", (int)brett[ i][ j]);
      else
        if( (int)brett[ i][ j] != 0) printf( "\tX");
        else  printf( "\t");
    printf( "\n\n");
  }
}

/*
 * Aufdecken aller Karten.
 */
void zeigeBrett( double **brett, int n)
{
  int i, j;

  for( j = 0; j < n; j++) printf( "\t%d", j);
  printf( "\n");
  for( j = 0; j < n; j++) printf( "\t_");
  printf( "\n\n");

  for( i = 0; i < n; i++)
  {
    printf( "%d | ", i);
    for( j = 0; j < n; j++)
      printf( "\t%d", (int)brett[ i][ j]);
    printf( "\n\n");
  }
}
